/***************************************************************************
                          k8000.c  -  description
                             -------------------
    begin                : Mon Jan 07 2002
    email                : joris@struyve.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: k8000.c,v 1.15 2002/04/01 11:14:13 js Exp $
 *
 */

#include <sys/io.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* Includes for I2C Kernel Device */
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/i2c.h>

#include "k8000.h"

#define DEBUG 0
#define HIGH 1
#define LOW 0

int deviceDescriptor;
char DeviceName[100]="/dev/velleman";

enum accessEnum { direct, dev};
enum accessEnum accessType;

typedef struct 
{
  short *scl_port_addr;
  short *sda_in_port_addr;
  short *sda_out_port_addr;
  
  byte scl_mask;
  byte sda_in_mask;
  byte sda_out_mask;
  
  byte scl_reverse;
  byte sda_in_reverse;
  byte sda_out_reverse;
} chain_config;

/*********************************************\
*  Pin assignment for  5 chains               *
***********************************************
* Chain 0 (default chain)                     *
* pin 17 1-Select in SCL D3 (controlport)     *
* pin 13 Select SDA in D4 (statusport)        *
* pin 14 1-Auto Feed SDA out D1 (controlport) *
***********************************************
* Chain 1                                     *
* pin 2 Data0 SCL D0 (dataport)               *
* pin 10 1-ACK SDA in D6 (statusport)         *
* pin 1 1-Strobe SDA out D0 (controlport)     *
***********************************************
* Chain 2                                     *
* pin 4 Data2 SCL D2 (dataport)               *
* pin 11 Busy SDA in D7 (statusport)          *
* pin 3 Data1 SDA out D1 (dataport)           *
***********************************************
* Chain 3                                     *
* pin 6 Data4 SCL D4 (dataport)               *
* pin 12 Paper Empty SDA in D5 (statusport)   *
* pin 5 Data3 SDA out D3 (dataport)           *
***********************************************
* Chain 4                                     *
* pin 8 Data6 SCL D6 (dataport)               *
* pin 15 1-Error SDA in D3 (statusport)       *
* pin 7 Data5 SDA out D5 (dataport)           *
\*********************************************/
short ControlPort;
short StatusPort;
short DataPort;

/* declare and init the chains array */
chain_config chains[5] = { {&ControlPort,&StatusPort ,&ControlPort,
                             0x08 /* 0000.1000 */,0x10 /* 0001.0000 */,0x02 /* 0000.0010 */,1,0,1},

                           {&DataPort   ,&StatusPort ,&ControlPort,
                             0x01 /* 0000.0001 */,0x40 /* 0100.0000 */,0x01 /* 0000.0001 */,0,1,1},

                           {&DataPort   ,&StatusPort , &DataPort,
                             0x04 /* 0000.0100 */,0x80 /* 1000.0000 */,0x02 /* 0000.0010 */,0,0,0},

                           {&DataPort   ,&StatusPort ,&DataPort,
                             0x10 /* 0001.0000 */,0x20 /* 0010.0000 */,0x08 /* 0000.1000 */,0,0,0},

                           {&DataPort   ,&StatusPort ,&DataPort,
                             0x40 /* 0100.0000 */,0x08 /* 0000.1000 */,0x20 /* 0010.0000 */,0,1,0}};


/* active_chain points to a chain_config struct in the chains array */
chain_config *active_chain = chains;

/* IC addresses ( 7 LSB's are meaningfull ) */
short IOchipcode[] = {0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F};
short DACchipcode[] = {0x20, 0x21, 0x22, 0x23};
short ADDAchipcode[] = {0x48, 0x49, 0x4A, 0x4B};

/* configuration and data matrix*/
short IOconfig_matrix[MaxChains][MaxIOchip+1];
short IOdata_matrix[MaxChains][MaxIOchip+1];
short IO_matrix[MaxChains][MaxIOchannel+1];
short DAC_matrix[MaxChains][MaxDACchannel+1];
short AD_matrix[MaxChains][MaxADchannel+1];
short DA_matrix[MaxChains][MaxDAchannel+1];

/* old configuration and data arrays  */
/* they just point to the first chain */
short *IOconfig = IOconfig_matrix[0];
short *IOdata = IOdata_matrix[0];
short *IO = IO_matrix[0];
short *DAC = DAC_matrix[0];
short *AD = AD_matrix[0];
short *DA = DA_matrix[0];

/* Global vars */
int I2CbusDelay = 1000;
int autoioperm=1;
int activechainnumber=0;

/**************************************************************\
*  IC basic functions (platform/interface specific)           *
*                                                              *
*  This is hte platform/interface specific part.               *
*  This is for user space progs over the parallel port.        *
*                                                              *
\**************************************************************/
inline void I2cDelay(void)
{
  int i,j;
  
  for (i=0;i<I2CbusDelay;i++) {j++;};
}

inline void SetScl(short state)
{
  I2cDelay();
  if ( (active_chain->scl_reverse && state) || (!active_chain->scl_reverse && !state) )
    outb(inb(*(active_chain->scl_port_addr)) & (~active_chain->scl_mask),
         *(active_chain->scl_port_addr) );
  else
    outb(inb(*(active_chain->scl_port_addr)) | active_chain->scl_mask,
         *(active_chain->scl_port_addr) );
    
}

inline void SetSda(short state)
{
  if ( (active_chain->sda_out_reverse && state) || (!active_chain->sda_out_reverse && !state) )
    outb(inb(*(active_chain->sda_out_port_addr)) & (~active_chain->sda_out_mask) ,
         *(active_chain->sda_out_port_addr));
  else
    outb(inb(*(active_chain->sda_out_port_addr)) | active_chain->sda_out_mask,
         *(active_chain->sda_out_port_addr));
}

inline int GetSda()
{
  if(active_chain->sda_in_reverse)
    return !((inb(*(active_chain->sda_in_port_addr))) & active_chain->sda_in_mask);
  else
    return ((inb(*(active_chain->sda_in_port_addr))) & active_chain->sda_in_mask);
}

/**************************************************************\
*  IC standard protocol conditions                            *
\**************************************************************/
inline void StartCond(void)
{
    /* Assert : SCL and SDA are high */
  SetSda(LOW);
  SetScl(LOW);
}

inline void I2CBusNotBusy(void)
{
  SetScl(HIGH);
  SetSda(HIGH);
}

inline void StopCond(void)
{
  SetSda(LOW);
  SetScl(LOW);
  SetScl(HIGH);
  SetSda(HIGH);
}

inline void ClockPulse(void)
{
  SetScl(LOW);
  SetScl(HIGH);
  SetScl(LOW);
}

inline void MasterAck(void)
{
  SetScl(LOW);
  SetSda(LOW);
  SetScl(HIGH);
  SetScl(LOW);
  SetSda(HIGH);
}

/**************************************************************\
*  Generic IC input/output functions                          *
\**************************************************************/
void I2cSendData_direct(byte addr,byte *data,int len)
{

  int i,j;
  int sb;
   
  /* set the read bit (0) */
  addr = (addr<<1);
   
  StartCond();
  if (DEBUG) printf("I2cSendData ADDR : ");
  /* send slave the address */
  for (i=7;i>=0;i--)
  {
    sb = addr & ( 1 << i );
    if (DEBUG) printf("%d",sb>0?1:0);
    if(sb)
      SetSda(HIGH);
    else
      SetSda(LOW);
    SetScl(HIGH);
    SetScl(LOW);
  }
  if (DEBUG) printf(" (DATA ");
  /* send the data */
  for (j=0;j<len;j++) /* for all data bytes */
  {
    if (DEBUG) printf(":");
    ClockPulse();
    for (i=7;i>=0;i--)
    {
      sb = (*(data +j)) & ( 1 << i );
      if (DEBUG) printf("%d",sb>0?1:0);
      if(sb)
        SetSda(HIGH);
      else
        SetSda(LOW);
      SetScl(HIGH);
      SetScl(LOW);
    }
  }
  if (DEBUG) printf(")\n");
  ClockPulse();
  StopCond();
}

void I2cSendData_device(byte addr,byte *data,int len)
{
  int result=0;
  if(ioctl(deviceDescriptor,I2C_SLAVE, addr))
    perror("LIBK8000 I2cSendData_device : IOCTL Problem");

  result=write(deviceDescriptor,data,len);
}

void I2cSendData(byte addr,byte *data,int len)
{

  switch(accessType)
  {
     case direct:
       I2cSendData_direct(addr,data,len);
       break;

     case dev:
       I2cSendData_device(addr,data,len);
       break;
  }
}

void I2cReadData_direct(byte addr,byte *data,int len)
{
  int i,j;
  int sb;

  /* set the write bit (1) */
  addr = (addr<<1)|1;
  
  StartCond();
  if (DEBUG) printf("I2cReadData ADDR : ");
  /* send slave the address */
  for (i=7;i>=0;i--)
  {
    sb = addr & ( 1 << i );
    if (DEBUG) printf("%d",sb>0?1:0);
    if(sb)
      SetSda(HIGH);
    else
      SetSda(LOW);
    SetScl(HIGH);
    SetScl(LOW);
        }
  if (DEBUG) printf(" (DATA ");
  
  /* read the data */
  for (j=0;j<len;j++) /* for all data bytes */
  {
      if (DEBUG) printf(":");
    if(!j)
      ClockPulse();
    else
      MasterAck();
    *(data+j) = 0;
    for (i=7;i>=0;i--)
    {
      SetScl(HIGH);
        sb = GetSda();
      if (DEBUG) printf("%d",sb>0?1:0);
      *(data+j) |= ((sb>0?1:0)<<i);
      SetScl(LOW);
          }
  }
  if (DEBUG) printf(")\n");
  ClockPulse();
  StopCond();
}

void I2cReadData_device(byte addr,byte *data,int len)
{
  int result=0;
  if(ioctl(deviceDescriptor,I2C_SLAVE, addr))
    perror("LIBK8000 I2cReadData_device : IOCTL Problem");

  result=read(deviceDescriptor,data,len);
}

void I2cReadData(byte addr,byte *data,int len)
{
  switch(accessType)
  {
     case direct:
       I2cReadData_direct(addr,data,len);
       break;

     case dev:
       I2cReadData_device(addr,data,len);
       break;
  }
}

/**************************************************************\
*  IO input procedures                                         *
\**************************************************************/
void ReadIOchip(int chip_no)
{
  short start_channel, channel;
  byte data;

  I2cReadData(IOchipcode[chip_no],&data,1);
  IOdata_matrix[activechainnumber][chip_no] = ~data;

  start_channel = chip_no * 8 + 1;
  for (channel = 0; channel <= 7; channel++)
    IO_matrix[activechainnumber][start_channel+channel] = ((IOdata_matrix[activechainnumber][chip_no] & (0x01 << channel)) != 0);
}

void ReadAllIO(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOchip; chip_no++)
    ReadIOchip(chip_no);
}

void ReadIOchannel(int channel_no)
{
  int chip_no;

  chip_no = (channel_no - 1) / 8;
  ReadIOchip(chip_no);
}

/**************************************************************\
*  IO output procedures                                        *
\**************************************************************/
void IOoutput(int chip_no, short data)
{
  byte start_channel, channel;
  byte tmp;

  tmp = (~data) | IOconfig_matrix[activechainnumber][chip_no];
  I2cSendData((byte)IOchipcode[chip_no],&tmp,1);

  IOdata_matrix[activechainnumber][chip_no] = (IOdata_matrix[activechainnumber][chip_no] & IOconfig_matrix[activechainnumber][chip_no]) | (~tmp);

  start_channel = chip_no * 8 + 1;
  for (channel = 0; channel <= 7; channel++)
    IO_matrix[activechainnumber][start_channel+channel] = ((IOdata_matrix[activechainnumber][chip_no] & (0x01 << channel)) != 0);
}

void ClearIOchip(int chip_no)
{ IOoutput(chip_no, 0); }

void ClearIOchannel(int channel_no)
{
  int chip_no, channel;
  byte data;

  chip_no = (channel_no - 1) / 8;
  channel = (channel_no - 1) % 8;
  data = IOdata_matrix[activechainnumber][chip_no] & (~(0x01 << channel));
  IOoutput(chip_no, data);
}

void SetIOchip(int chip_no)
{
  IOoutput(chip_no, 0xFF);
}

void SetIOchannel(int channel_no)
{
  int chip_no, channel;
  byte data;

  chip_no = (channel_no - 1) / 8;
  channel = (channel_no - 1) % 8;
  data = IOdata_matrix[activechainnumber][chip_no] | (0x01 << channel);
  IOoutput(chip_no, data);
}

void SetAllIO(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOchip; chip_no++)
    IOoutput(chip_no, 0xFF);
}

void ClearAllIO(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOchip; chip_no++)
    IOoutput(chip_no, 0);
}

/**************************************************************\
*  IO data update procedures                                   *
\**************************************************************/
void UpdateIOdataArray(int chip_no, short data)
{
  int start_channel, channel;

  IOdata_matrix[activechainnumber][chip_no] = (IOdata_matrix[activechainnumber][chip_no] & IOconfig_matrix[activechainnumber][chip_no]);
  IOdata_matrix[activechainnumber][chip_no] = IOdata_matrix[activechainnumber][chip_no] | (data & (~IOconfig_matrix[activechainnumber][chip_no]));
  
  start_channel = chip_no * 8 + 1;
  for (channel = 0; channel <= 7; channel++)
    IO_matrix[activechainnumber][start_channel+channel] = ((IOdata_matrix[activechainnumber][chip_no] & (0x01 << channel)) != 0);
}

void ClearIOdataArray(int chip_no)
{
  int start_channel, channel;

  IOdata_matrix[activechainnumber][chip_no] = IOdata_matrix[activechainnumber][chip_no] & IOconfig_matrix[activechainnumber][chip_no];
  start_channel = chip_no * 8 + 1;
  for (channel = 0; channel <= 7; channel++)
    IO_matrix[activechainnumber][start_channel+channel] = ((IOdata_matrix[activechainnumber][chip_no] & (0x01 << channel)) != 0);
}

void SetIOdataArray(int chip_no)
{
  int start_channel, channel;

  IOdata_matrix[activechainnumber][chip_no] = IOdata_matrix[activechainnumber][chip_no] | ( ~IOconfig_matrix[activechainnumber][chip_no]);
  start_channel = chip_no * 8 + 1;
  for (channel = 0; channel <= 7; channel++)
    IO_matrix[activechainnumber][start_channel+channel] = ((IOdata_matrix[activechainnumber][chip_no] & (0x01 << channel)) != 0);
}

void SetIOchArray(int channel_no)
{
  int chip_no, channel;
  byte data;

  chip_no = (channel_no - 1) / 8;
  channel = (channel_no - 1) % 8;
  data = IOdata_matrix[activechainnumber][chip_no] | (0x01 << channel);
  UpdateIOdataArray(chip_no, data);
}

void ClearIOchArray(int channel_no)
{
  int chip_no, channel;
  byte data;

  chip_no = (channel_no - 1) / 8;
  channel = (channel_no - 1) % 8;
  data = IOdata_matrix[activechainnumber][chip_no] & (~(0x01 << channel));
  UpdateIOdataArray(chip_no, data);
}

void UpdateAllIO(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOchip; chip_no++)
    IOoutput(chip_no, IOdata_matrix[activechainnumber][chip_no]);
}

void UpdateIOchip(int chip_no)
{ IOoutput(chip_no, IOdata_matrix[activechainnumber][chip_no]); }


/**************************************************************\
*  IO config procedures                                        *
\**************************************************************/
void ConfigAllIOasInput(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOchip; chip_no++)
  {
    IOconfig_matrix[activechainnumber][chip_no] = 0;
    ClearIOchip(chip_no);
    IOconfig_matrix[activechainnumber][chip_no] = 0xFF;
    ReadIOchip(chip_no);
  }
}

void ConfigAllIOasOutput(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOchip; chip_no++)
    IOconfig_matrix[activechainnumber][chip_no] = 0x00;
  ClearAllIO();
}

void ConfigIOchipAsInput(int chip_no)
{
  IOconfig_matrix[activechainnumber][chip_no] = 0;
  ClearIOchip(chip_no);
  IOconfig_matrix[activechainnumber][chip_no] = 0xFF;
  ReadIOchip(chip_no);
}

void ConfigIOchipAsOutput(int chip_no)
{
  IOconfig_matrix[activechainnumber][chip_no] = 0x00;
  ClearIOchip(chip_no);
}

void ConfigIOchannelAsInput(int channel_no)
{
  int chip_no, channel;

  chip_no = (channel_no - 1) / 8;
  channel = (channel_no - 1) % 8;
  IOconfig_matrix[activechainnumber][chip_no] = IOconfig_matrix[activechainnumber][chip_no] & (~(0x01 << channel));
  ClearIOchannel(channel_no);
  IOconfig_matrix[activechainnumber][chip_no] = IOconfig_matrix[activechainnumber][chip_no] | (0x01 << channel);
  ReadIOchannel(channel_no);
}

void ConfigIOchannelAsOutput(int channel_no)
{
  int chip_no, channel;

  chip_no = (channel_no - 1) / 8;
  channel = (channel_no - 1) % 8;
  IOconfig_matrix[activechainnumber][chip_no] = IOconfig_matrix[activechainnumber][chip_no] & (~(0x01 << channel));
  ClearIOchannel(channel_no);
}

/**************************************************************\
*  DAC output procedures                                       *
\**************************************************************/
void OutputDACchannel(int channel_no, int data)
{
  byte total_data[2];

  total_data[0] = (0xF0 | ((channel_no - 1) % 8));
  total_data[1] = (data > 63) ? 63: data;
  DAC_matrix[activechainnumber][channel_no] = total_data[1];
  
  I2cSendData(DACchipcode[(channel_no-1)/8],total_data,2);
}

void UpdateDACchip(int chip_no)
{
  byte total_data[9];
  int i,channel;

  total_data[0] = 0x00;
  channel = chip_no * 8;
  for(i=1;i<=8;i++)
    total_data[i] = DAC_matrix[activechainnumber][channel + i] = (DAC_matrix[activechainnumber][channel+i] > 63) ? 63 : DAC_matrix[activechainnumber][channel+i];

  I2cSendData(DACchipcode[chip_no],total_data,9);
}

void UpdateDACchannel(int channel_no)
{ OutputDACchannel(channel_no, DAC_matrix[activechainnumber][channel_no]); }

void ClearDACchannel(int channel_no)
{ OutputDACchannel(channel_no, 0); }

void SetDACchannel(int channel_no)
{ OutputDACchannel(channel_no,63); }

void ClearDACchip(int chip_no)
{
  int i, channel;

  channel = chip_no * 8;
  for (i = 1; i <= 8; i++)
    DAC_matrix[activechainnumber][channel + i] = 0;
  UpdateDACchip(chip_no);
}

void SetDACchip(int chip_no)
{
  int i, channel;

  channel = chip_no * 8;
  for (i = 1; i <= 8; i++)
    DAC_matrix[activechainnumber][channel + i] = 63;
  UpdateDACchip(chip_no);
}

void UpdateAllDAC(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOcard; chip_no++)
    UpdateDACchip(chip_no);
}

void ClearAllDAC(void)
{
  int channel_no;

  for (channel_no = 1; channel_no <= MaxDACchannel; channel_no++)
    DAC_matrix[activechainnumber][channel_no] = 0x00;
  UpdateAllDAC();
}

void SetAllDAC(void)
{
  int channel_no;

  for (channel_no = 1; channel_no <= MaxDACchannel; channel_no++)
    DAC_matrix[activechainnumber][channel_no] = 63;
  UpdateAllDAC();
}

/**************************************************************\
*  DA output procedures                                        *
\**************************************************************/
void OutputDAchannel(int channel_no, int data)
{
  byte total_data[2];

  total_data[0] = (0x40);
  total_data[1] = DA_matrix[activechainnumber][channel_no] = data;
  
  I2cSendData(ADDAchipcode[channel_no-1],total_data,2);
}

void UpdateDAchannel(int channel_no)
{ OutputDAchannel(channel_no, DA_matrix[activechainnumber][channel_no]); }

void ClearDAchannel(int channel_no)
{ OutputDAchannel(channel_no, 0); }

void SetDAchannel(int channel_no)
{ OutputDAchannel(channel_no, 255); }

void UpdateAllDA(void)
{
  int channel_no;

  for (channel_no = 1; channel_no <= MaxDAchannel; channel_no++)
    OutputDAchannel(channel_no, DA_matrix[activechainnumber][channel_no]);
}

void ClearAllDA(void)
{
  int channel_no;

  for (channel_no = 1; channel_no <= MaxDAchannel; channel_no++)
    OutputDAchannel(channel_no, 0);
}

void SetAllDA(void)
{
  int channel_no;

  for (channel_no = 1; channel_no <= MaxDAchannel; channel_no++ )
    OutputDAchannel(channel_no, 255);
}

/**************************************************************\
*  AD input procedures                                         *
\**************************************************************/
void ReadADchannel(int channel_no)
{
  byte total_data[2];

  total_data[0] = (0x40 | ((channel_no - 1) % 4));
  I2cSendData(ADDAchipcode[(channel_no-1)/4],total_data,1);

  I2cReadData(ADDAchipcode[(channel_no-1)/4],total_data,2);
  AD_matrix[activechainnumber][channel_no] = total_data[1];
}

void ReadADchip(int chip_no)
{
  byte total_data[5];
  int channel;

  total_data[0] = (0x44);
  I2cSendData(ADDAchipcode[chip_no],total_data,1);

  I2cReadData(ADDAchipcode[chip_no],total_data,5);

  channel = chip_no * 4 + 1;
  AD_matrix[activechainnumber][channel] = total_data[1];
  AD_matrix[activechainnumber][channel + 1] = total_data[2];
  AD_matrix[activechainnumber][channel + 2] = total_data[3];
  AD_matrix[activechainnumber][channel + 3] = total_data[4];
}

void ReadAllAD(void)
{
  int chip_no;

  for (chip_no = 0; chip_no <= MaxIOcard; chip_no++)
    ReadADchip(chip_no);
}

/**************************************************************\
*  Complete card and general procedures                        *
\**************************************************************/
void ReadAll(void)
{
  ReadAllIO();
  ReadAllAD();
}

void ReadCard(int card_no)
{
  int chip_no;

  chip_no = card_no * 2;
  ReadIOchip(chip_no);
  ReadIOchip(chip_no + 1);
  ReadADchip(card_no);
}

void UpdateAll(void)
{
  UpdateAllIO();
  UpdateAllDAC();
  UpdateAllDA();
}

void UpdateCard(int card_no)
{
  int chip_no;

  chip_no = card_no * 2;
  UpdateIOchip(chip_no);
  UpdateIOchip(chip_no + 1);
  UpdateDACchip(card_no);
  UpdateDAchannel(card_no + 1);
}

void DecToBin(int DecNumber, char *Bin)
{
  int i;
  char *one = "1", *zero = "0";

  DecNumber <<= 8;
  if (DecNumber < 0)
    strcpy(Bin, one);
  else
    strcpy(Bin, zero);
  DecNumber <<= 1;
  for (i=1; i<8; i++)
  {
    if (DecNumber < 0)
      strcat(Bin, one);
    else
      strcat(Bin, zero);
    DecNumber <<= 1;
  }
}

int BinToDec(char *BinNumber)
{
  int i, Weight, dec;

  Weight = 1;
  dec = 0;
  for (i=strlen(BinNumber)-1;i>=0;i--)
  {
    dec = dec + (BinNumber[i] == '1') * Weight;
    Weight = Weight << 1;
  }
  return dec;
}

void DecToHex(int DecNumber, char *Hex)
{
  strcpy(Hex,"0x");
  /*itoa(DecNumber / 16, Hex+2, 16);*/
  /*itoa(DecNumber % 16, Hex+3, 16);*/
}

int HexToDec(char *HexNumber)
{
  int Strlength, dech = 0, decl;

  Strlength = strlen(HexNumber);
  switch(HexNumber[Strlength-1])
  {
    case 'a': case 'A': decl = 10; break;
    case 'b': case 'B': decl = 11; break;
    case 'c': case 'C': decl = 12; break;
    case 'd': case 'D': decl = 13; break;
    case 'e': case 'E': decl = 14; break;
    case 'f': case 'F': decl = 15; break;
    default: decl = HexNumber[Strlength-1]-'0';
  }
  if (Strlength == 2)
  {
    switch(HexNumber[0])
    {
      case 'a': case 'A': dech = 10; break;
      case 'b': case 'B': dech = 11; break;
      case 'c': case 'C': dech = 12; break;
      case 'd': case 'D': dech = 13; break;
      case 'e': case 'E': dech = 14; break;
      case 'f': case 'F': dech = 15; break;
      default: dech = HexNumber[0]-'0';
    }
  }
  return ((dech << 4) + decl);
}

int SelectI2CprinterPort(int Printer_no)
{

  switch (Printer_no)
  {

    case I2C_DEV: /*  Use I2C Kernel Device ( -1 = I2C_DEV) */
      accessType=dev;
      deviceDescriptor=open(DeviceName, O_RDWR);
      if (deviceDescriptor == -1)
      {
	printf("LIBK8000: Error opening device '%s'\n",strerror(errno));
	return -1;
      }
      else
      	return 0;
      break;

    case 0: /* lpt on monochroom display adapter */
      accessType=direct;
      DataPort    = 0x03BC;
      StatusPort  = 0x03BD;
      ControlPort = 0x03BE;
      if (autoioperm)
        return ioperm(0x3BC, 3, 1);
      break;

    case 1:  /* lpt1 on mainboard */
      accessType=direct;
      DataPort    = 0x0378;
      StatusPort  = 0x0379;
      ControlPort = 0x037A;
      if (autoioperm)
        return ioperm(0x378, 3, 1);
      break;

    case 2:  /* lpt2 on mainboard */
      accessType=direct;
      DataPort    = 0x0278;
      StatusPort  = 0x0279;
      ControlPort = 0x027A;
      if (autoioperm)
        return ioperm(0x278, 3, 1);
      break;
  }
  return 0;
}

void SelectChain(int chain_no)
{
  if(accessType==dev){
    if(chain_no!=0){
      printf("LIBK8000: SelectChain: ERROR: Chains 1, 2, 3 and 4 can not (yet ?) be used with i2c device\n");
      printf("                              Only default chain 0 can be used with i2c device. \n");
      printf("                              Use direct parallel port instead.\n\n");
      exit(0);
    }
  }
  if (chain_no>4) chain_no = 4;
    active_chain = chains+(chain_no);
  activechainnumber = chain_no;
}

void initialize()
{
  SelectI2CprinterPort(1);
  I2CbusDelay = 1;
  I2CBusNotBusy();
}

void SetAutoPerm(int set)
{ autoioperm = set; }
